# NOTE: Derived from blib/lib/Text/EP3/Verilog.pm.
# Changes made here will be lost when autosplit is run again.
# See AutoSplit.pm.
package Text::EP3::Verilog;

#line 146 "blib/lib/Text/EP3/Verilog.pm (autosplit into blib/lib/auto/Text/EP3/Verilog/signal.al)"
sub signal {
# Signal replacement method.
# usage: @signal signame | signame[width1:width2], etc. 
# Take a list of signals and generate signal lists in the differing formats
# that Verilog uses.
# This is accomplished by formatting a list of new defines and then calling
# the EP3 define method
#   @signal KEY a[3:0], b, c[width:0], etc.
# and 
# Replace KEY with the list as it appears (can be used in further signal defs)
# Replace KEY{SIG} with the signal list (can be used in port lists)
#    e.g. replace KEY{SIG} with  a[3:0], b, c[width:0]
# Replace KEY{EVENT} with the reg list  (To be used in event lists)
#    e.g. replace KEY{EVENT} with a or b or c
# Replace KEY{IN}  with the input list (you supply the first input and the trailing ';'
#    e.g. replace KEY{INPUT} with [3:0] a;\ninput b;\ninput[width:0] c
#    or ... make the line 
#    input KEY{INPUT}; become ..
#    input [3:0] a;
#    input b;
#    input [width:0] c;
# Replace KEY{OUT} with the output list (output [] sig).
#    e.g. like KEY{IN}
# Replace KEY{INOUT}  with the inout list (inout [] sig).
#    e.g. like KEY{IN}
# Replace KEY{WIRE} with the wire list (wire [] sig).
#    e.g. like KEY{IN}
# Replace KEY{REG} with the reg list (reg [] sig).
#    e.g. like KEY{IN}
# Replace KEY{DSP} with the printf list (sig=%0[b|x] depending on width).
#    e.g. replace KEY{DSP} with a=%0x, b=%0b, c=%0x
#    This can be used in the $display task
#       $display("KEY{DSP}",KEY{SIG});
#
    my $self  = shift;
    my @input_string = @_;
    my ($inline, $directive, $key);
    my ($busstring , $sigstring , $dspstring , $instring , $outstring , $inoutstring , $wirestring , $regstring , $eventstring );
    my (@string, $signals, $reg, $input, $output, $inout, $event, @siglist, $sig, $bus, $newkey);
 
    $busstring = $sigstring = $dspstring = $instring = $outstring = $inoutstring = $wirestring = $regstring = $eventstring  = '';
 
    $inline = $input_string[0];
    @string = split(' ',$inline);
 
    print "$self->{Line_Comment}EP3->signal: Entered signal.  Line $Text::EP3::line of $Text::EP3::filename  The key is $string[1]\n"	if $self->{Debug} & 1;
 
    # parse key string
    $directive = shift @string;
    $key = shift @string;
    $signals =  join(' ',@string);
 
    my $wire = $reg = $input = $output = $inout =  $event = ''; #Empty the first time through
  
    $signals =~ s/ //g;
    @siglist = split(',',$signals);
    foreach $sig (@siglist)
    {
        next if ($sig =~/^\s*$/); # skip it if it is a blank - could have been preprocessed away
        if ($sig =~ /(\[.*\])/)    # if it is a bus
        {
            $bus = $1;
            $bus =~ s/ //g;
            $sig  =~ s/\[.*\]//;
            $dspstring.="$sig=%0x,";
            $wirestring.="$wire$bus $sig;\n";
            $regstring.="$reg$bus $sig;\n";
            $instring.="$input$bus $sig;\n";
            $inoutstring.="$inout$bus $sig;\n";
            $outstring.="$output$bus $sig;\n";
        }
        else
        {
            $dspstring.="$sig=%0b,";
            $wirestring.="$wire$sig;\n";
            $regstring.="$reg$sig;\n";
            $instring.="$input$sig;\n";
            $inoutstring.="$inout$sig;\n";
            $outstring.="$output$sig;\n";
        }
  
        $sigstring.="$sig,";
        $eventstring.="$event$sig";
  
        $input = 'input ';
        $output = 'output ';
        $inout = 'inout ';
        $wire = 'wire ';
        $reg = 'reg ';
        $event = ' or ';
    }
 
    #remove trailing semicolon
    $sigstring =~ s/,$//;
    $dspstring =~ s/,$//;
    $instring =~ s/;\n$//;
    $inoutstring =~ s/;\n$//;
    $outstring =~ s/;\n$//;
    $wirestring =~ s/;\n$//;
    $regstring =~ s/;\n$//;
 
    $directive = $self->{Delimeter} . "define";
    
    $newkey = $key."{SIG}";
    $sigstring = $directive.' '.$newkey.' '.$sigstring;
    $self->define($sigstring);
 
    $newkey = $key."{DSP}";
    $dspstring = $directive.' '.$newkey.' '.$dspstring;
    $self->define($dspstring);
 
    $newkey = $key."{IN}";
    $instring = $directive.' '.$newkey.' '.$instring;
    $self->define($instring);
 
    $newkey = $key."{OUT}";
    $outstring = $directive.' '.$newkey.' '.$outstring;
    $self->define($outstring);
 
    $newkey = $key."{INOUT}";
    $inoutstring = $directive.' '.$newkey.' '.$inoutstring;
    $self->define($inoutstring);
 
    $newkey = $key."{WIRE}";
    $wirestring = $directive.' '.$newkey.' '.$wirestring;
    $self->define($wirestring);
 
    $newkey = $key."{REG}";
    $regstring = $directive.' '.$newkey.' '.$regstring;
    $self->define($regstring);
 
    $newkey = $key."{EVENT}";
    $eventstring = $directive.' '.$newkey.' '.$eventstring;
    $self->define($eventstring);
 
    # Define the key as the list with the define directive
    # Do this last to avoid conflicts with the preceding defines.
    $self->define($directive.' '.$key.' '.$signals);
}

# end of Text::EP3::Verilog::signal
1;
